import tkinter as tk
from tkinter import ttk

from thonny import get_workbench
from thonny.config_ui import ConfigurationPage


class GeneralConfigurationPage(ConfigurationPage):
    def __init__(self, master):
        ConfigurationPage.__init__(self, master)

        self.add_checkbox(
            "general.single_instance",
            "Allow only single Thonny instance",
            row=1,
            columnspan=2,
        )
        self.add_checkbox(
            "general.debug_mode",
            "Debug mode (provides more detailed logs)",
            row=3,
            columnspan=2,
        )
        self.add_checkbox(
            "file.reopen_all_files",
            "Reopen all files from previous session",
            row=4,
            columnspan=2,
        )
        self.add_checkbox(
            "general.disable_notification_sound",
            "Disable notification sound",
            row=5,
            columnspan=2,
        )

        ttk.Label(self, text="UI mode").grid(row=6, column=0, sticky=tk.W, padx=(0, 10), pady=(10, 0))
        self.add_combobox(
            "general.ui_mode",
            ["simple", "regular", "expert"],
            row=6,
            column=1,
            pady=(10, 0)
        )

        self._scaling_var = get_workbench().get_variable("general.scaling")
        self._scaling_label = ttk.Label(self, text="UI scaling factor")
        self._scaling_label.grid(
            row=7, column=0, sticky=tk.W, padx=(0, 10), pady=(10, 0)
        )
        scalings = sorted({0.5, 0.75, 1.0, 1.25, 1.33, 1.5, 2.0, 2.5, 3.0, 4.0})
        self._scaling_combo = ttk.Combobox(
            self,
            width=7,
            exportselection=False,
            textvariable=self._scaling_var,
            state="readonly",
            height=15,
            values=["default"] + scalings,
        )
        self._scaling_combo.grid(row=7, column=1, sticky=tk.W, pady=(10, 0))


        self._font_scaling_var = get_workbench().get_variable("general.font_scaling_mode")
        self._font_scaling_label = ttk.Label(self, text="Font scaling mode")
        self._font_scaling_label.grid(
            row=8, column=0, sticky=tk.W, padx=(0, 10), pady=(10, 0)
        )
        scalings = sorted({0.5, 0.75, 1.0, 1.25, 1.33, 1.5, 2.0, 2.5, 3.0, 4.0})
        self._font_scaling_combo = ttk.Combobox(
            self,
            width=10,
            exportselection=False,
            textvariable=self._font_scaling_var,
            state="readonly",
            height=15,
            values=["default", "extra", "automatic"],
        )
        self._font_scaling_combo.grid(row=8, column=1, sticky=tk.W, pady=(10, 0))

        reopen_label = ttk.Label(
            self,
            text="NB! Restart Thonny after changing these options"
            + "\nin order to see the full effect",
            font="BoldTkDefaultFont",
        )
        reopen_label.grid(row=20, column=0, sticky=tk.W, pady=20, columnspan=2)

        self.columnconfigure(1, weight=1)


def load_plugin() -> None:
    get_workbench().add_configuration_page("General", GeneralConfigurationPage)



import os
from thonny.backend import get_vm, Executor, prepare_hooks, return_execution_result
import webbrowser


def _cmd_Birdseye(cmd):
    vm = get_vm()
    vm.switch_env_to_script_mode(cmd)
    return vm._execute_file(cmd, BirdsEyeRunner)


class BirdsEyeRunner(Executor):
    @return_execution_result
    @prepare_hooks
    def execute_source(self, source, filename, mode, ast_postprocessors):
        assert mode == "exec"
        # ignore ast_postprocessors, because birdseye requires source

        if isinstance(source, bytes):
            source = source.decode("utf-8")

        import __main__  # @UnresolvedImport
        global_vars = __main__.__dict__

        # Following is a trick, which allows importing birdseye in the backends,
        # which doesn't have it installed (provided it is installed for frontend Python)
        self._vm.load_modules_with_frontend_path(["birdseye.bird"])
        from birdseye.bird import eye

        eye.exec_string(source, filename, globs=global_vars, locs=global_vars, deep=True)
        port = os.environ.get("BIRDSEYE_PORT", "7777")
        webbrowser.open_new_tab('http://localhost:%s/ipython_call/' % port
                                + eye._last_call_id)


def load_plugin():
    get_vm().add_command("Birdseye", _cmd_Birdseye)



import textwrap
import tkinter as tk
from tkinter import font as tk_font
from tkinter import ttk

from thonny import get_workbench
from thonny.codeview import CodeView
from thonny.config_ui import ConfigurationPage
from thonny.ui_utils import create_string_var


class ThemeAndFontConfigurationPage(ConfigurationPage):
    def __init__(self, master):

        self._original_family = get_workbench().get_option("view.editor_font_family")
        self._original_size = get_workbench().get_option("view.editor_font_size")
        self._original_ui_theme = get_workbench().get_option("view.ui_theme")
        self._original_syntax_theme = get_workbench().get_option("view.syntax_theme")

        ConfigurationPage.__init__(self, master)

        self._family_variable = create_string_var(
            self._original_family, modification_listener=self._update_appearance
        )

        self._size_variable = create_string_var(
            self._original_size, modification_listener=self._update_appearance
        )

        self._ui_theme_variable = create_string_var(
            self._original_ui_theme, modification_listener=self._update_appearance
        )

        self._syntax_theme_variable = create_string_var(
            self._original_syntax_theme, modification_listener=self._update_appearance
        )

        ttk.Label(self, text="UI theme").grid(row=1, column=0, sticky="w", pady=(10, 0))
        self._ui_theme_combo = ttk.Combobox(
            self,
            exportselection=False,
            textvariable=self._ui_theme_variable,
            state="readonly",
            height=15,
            values=get_workbench().get_usable_ui_theme_names(),
        )
        self._ui_theme_combo.grid(row=2, column=0, sticky="nsew", padx=(0, 10))

        ttk.Label(self, text="Syntax theme").grid(
            row=1, column=1, sticky="w", pady=(10, 0)
        )
        self._syntax_theme_combo = ttk.Combobox(
            self,
            exportselection=False,
            textvariable=self._syntax_theme_variable,
            state="readonly",
            height=15,
            values=get_workbench().get_syntax_theme_names(),
        )
        self._syntax_theme_combo.grid(row=2, column=1, sticky="nsew", padx=(0, 10))

        ttk.Label(self, text="Editor font").grid(
            row=1, column=2, sticky="w", pady=(10, 0)
        )
        self._family_combo = ttk.Combobox(
            self,
            exportselection=False,
            state="readonly",
            height=15,
            textvariable=self._family_variable,
            values=self._get_families_to_show(),
        )
        self._family_combo.grid(row=2, column=2, sticky=tk.NSEW, padx=(0, 10))

        ttk.Label(self, text="Size").grid(row=1, column=3, sticky="w", pady=(10, 0))
        self._size_combo = ttk.Combobox(
            self,
            width=4,
            exportselection=False,
            textvariable=self._size_variable,
            state="readonly",
            height=15,
            values=[str(x) for x in range(3, 73)],
        )
        self._size_combo.grid(row=2, column=3, sticky="nsew")

        ttk.Label(self, text="Editor preview").grid(
            row=3, column=0, sticky="w", pady=(10, 0), columnspan=4
        )
        self._preview_codeview = CodeView(
            self,
            height=10,
            font="EditorFont",
            # relief="sunken",
            # borderwidth=1,
        )

        self._preview_codeview.set_content(
            textwrap.dedent(
                """
            def foo(bar):
                if bar is None: # This is a comment
                    print("The answer is", 33)
            
            unclosed_string = "blah, blah
            """
            ).strip()
        )
        self._preview_codeview.grid(
            row=4, column=0, columnspan=4, sticky=tk.NSEW, pady=(0, 5)
        )

        (
            ttk.Label(
                self, text="NB! Some style elements change only after restarting Thonny"
            ).grid(row=5, column=0, columnspan=4, sticky="w", pady=(0, 5))
        )

        self.columnconfigure(0, weight=1)
        self.columnconfigure(1, weight=1)
        self.columnconfigure(2, weight=1)
        self.columnconfigure(3, weight=1)
        self.rowconfigure(4, weight=1)

    def apply(self):
        # don't do anything, as preview already did the thing
        return

    def cancel(self):
        if (
            getattr(self._family_variable, "modified")
            or getattr(self._size_variable, "modified")
            or getattr(self._ui_theme_variable, "modified")
            or getattr(self._syntax_theme_variable, "modified")
        ):
            get_workbench().set_option("view.ui_theme", self._original_ui_theme)
            get_workbench().set_option("view.syntax_theme", self._original_syntax_theme)
            get_workbench().set_option("view.editor_font_size", self._original_size)
            get_workbench().set_option("view.editor_font_family", self._original_family)
            get_workbench().reload_themes()
            get_workbench().update_fonts()

    def _update_appearance(self):
        get_workbench().set_option("view.ui_theme", self._ui_theme_variable.get())
        get_workbench().set_option(
            "view.syntax_theme", self._syntax_theme_variable.get()
        )
        get_workbench().set_option(
            "view.editor_font_size", int(self._size_variable.get())
        )
        get_workbench().set_option(
            "view.editor_font_family", self._family_variable.get()
        )
        get_workbench().reload_themes()
        get_workbench().update_fonts()

    def _get_families_to_show(self):
        # In Linux, families may contain duplicates (actually different fonts get same names)
        return sorted(set(filter(lambda name: name[0].isalpha(), tk_font.families())))


def load_plugin() -> None:
    get_workbench().add_configuration_page(
        "Theme & Font", ThemeAndFontConfigurationPage
    )



# -*- coding: utf-8 -*-

import io
import tkinter as tk
import tokenize
from typing import Dict, Union  # @UnusedImport

from thonny import get_workbench, roughparse, tktextext, ui_utils
from thonny.common import TextRange
from thonny.misc_utils import running_on_windows
from thonny.tktextext import EnhancedText
from thonny.ui_utils import EnhancedTextWithLogging, scrollbar_style

_syntax_options = {}  # type: Dict[str, Union[str, int]]
# BREAKPOINT_SYMBOL = "•" # Bullet
# BREAKPOINT_SYMBOL = "○" # White circle
BREAKPOINT_SYMBOL = "●"  # Black circle


class SyntaxText(EnhancedText):
    def __init__(self, master=None, cnf={}, **kw):
        self._syntax_options = {}
        super().__init__(master=master, cnf=cnf, **kw)
        self._syntax_theme_change_binding = tk._default_root.bind(
            "<<SyntaxThemeChanged>>", self._reload_syntax_options, True
        )
        self._reload_syntax_options()

    def set_syntax_options(self, syntax_options):
        # clear old options
        for tag_name in self._syntax_options:
            self.tag_reset(tag_name)

        # apply new options
        for tag_name in syntax_options:
            if tag_name == "TEXT":
                self.configure(**syntax_options[tag_name])
            else:
                self.tag_configure(tag_name, **syntax_options[tag_name])

        self._syntax_options = syntax_options

        if "current_line" in syntax_options:
            self.tag_lower("current_line")

        self.tag_raise("sel")

    def _reload_theme_options(self, event=None):
        super()._reload_theme_options(event)
        self._reload_syntax_options(event)

    def _reload_syntax_options(self, event=None):
        global _syntax_options
        self.set_syntax_options(_syntax_options)

    def destroy(self):
        super().destroy()
        tk._default_root.unbind(
            "<<SyntaxThemeChanged>>", self._syntax_theme_change_binding
        )


class PythonText(SyntaxText):
    def __init__(self, master=None, cnf={}, **kw):
        if "indent_with_tabs" not in kw:
            kw["indent_with_tabs"] = False
            
        super().__init__(master=master, cnf=cnf, **kw)
    
    def perform_return(self, event):
        # copied from idlelib.EditorWindow (Python 3.4.2)
        # slightly modified
        # pylint: disable=lost-exception

        text = event.widget
        assert text is self

        try:
            # delete selection
            first, last = text.get_selection_indices()
            if first and last:
                text.delete(first, last)
                text.mark_set("insert", first)

            # Strip whitespace after insert point
            # (ie. don't carry whitespace from the right of the cursor over to the new line)
            while text.get("insert") in [" ", "\t"]:
                text.delete("insert")

            left_part = text.get("insert linestart", "insert")
            # locate first non-white character
            i = 0
            n = len(left_part)
            while i < n and left_part[i] in " \t":
                i = i + 1

            # is it only whitespace?
            if i == n:
                # start the new line with the same whitespace
                text.insert("insert", "\n" + left_part)
                return "break"

            # Turned out the left part contains visible chars
            # Remember the indent
            indent = left_part[:i]

            # Strip whitespace before insert point
            # (ie. after inserting the linebreak this line doesn't have trailing whitespace)
            while text.get("insert-1c", "insert") in [" ", "\t"]:
                text.delete("insert-1c", "insert")

            # start new line
            text.insert("insert", "\n")

            # adjust indentation for continuations and block
            # open/close first need to find the last stmt
            lno = tktextext.index2line(text.index("insert"))
            y = roughparse.RoughParser(text.indent_width, text.tabwidth)

            for context in roughparse.NUM_CONTEXT_LINES:
                startat = max(lno - context, 1)
                startatindex = repr(startat) + ".0"
                rawtext = text.get(startatindex, "insert")
                y.set_str(rawtext)
                bod = y.find_good_parse_start(
                    False, roughparse._build_char_in_string_func(startatindex)
                )
                if bod is not None or startat == 1:
                    break
            y.set_lo(bod or 0)

            c = y.get_continuation_type()
            if c != roughparse.C_NONE:
                # The current stmt hasn't ended yet.
                if c == roughparse.C_STRING_FIRST_LINE:
                    # after the first line of a string; do not indent at all
                    pass
                elif c == roughparse.C_STRING_NEXT_LINES:
                    # inside a string which started before this line;
                    # just mimic the current indent
                    text.insert("insert", indent)
                elif c == roughparse.C_BRACKET:
                    # line up with the first (if any) element of the
                    # last open bracket structure; else indent one
                    # level beyond the indent of the line with the
                    # last open bracket
                    text._reindent_to(y.compute_bracket_indent())
                elif c == roughparse.C_BACKSLASH:
                    # if more than one line in this stmt already, just
                    # mimic the current indent; else if initial line
                    # has a start on an assignment stmt, indent to
                    # beyond leftmost =; else to beyond first chunk of
                    # non-whitespace on initial line
                    if y.get_num_lines_in_stmt() > 1:
                        text.insert("insert", indent)
                    else:
                        text._reindent_to(y.compute_backslash_indent())
                else:
                    assert 0, "bogus continuation type %r" % (c,)
                return "break"

            # This line starts a brand new stmt; indent relative to
            # indentation of initial line of closest preceding
            # interesting stmt.
            indent = y.get_base_indent_string()
            text.insert("insert", indent)
            if y.is_block_opener():
                text.perform_smart_tab(event)
            elif indent and y.is_block_closer():
                text.perform_smart_backspace(event)
            return "break"
        finally:
            text.see("insert")
            text.event_generate("<<NewLine>>")
            return "break"


class CodeViewText(EnhancedTextWithLogging, PythonText):
    """Provides opportunities for monkey-patching by plugins"""

    def __init__(self, master=None, cnf={}, **kw):
        if "replace_tabs" not in kw:
            kw["replace_tabs"] = True

        super().__init__(
            master=master,
            tag_current_line=get_workbench().get_option("view.highlight_current_line"),
            cnf=cnf,
            **kw
        )
        # Allow binding to events of all CodeView texts
        self.bindtags(self.bindtags() + ("CodeViewText",))
        tktextext.fixwordbreaks(tk._default_root)

    def on_secondary_click(self, event=None):
        super().on_secondary_click(event)
        self.mark_set("insert", "@%d,%d" % (event.x, event.y))

        menu = get_workbench().get_menu("edit")
        try:
            from thonny.plugins.debugger import get_current_debugger

            debugger = get_current_debugger()
            if debugger is not None:
                menu = debugger.get_editor_context_menu()
        except ImportError:
            pass

        menu.tk_popup(event.x_root, event.y_root)


class CodeView(tktextext.TextFrame):
    def __init__(self, master, propose_remove_line_numbers=False, **text_frame_args):

        tktextext.TextFrame.__init__(
            self,
            master,
            text_class=CodeViewText,
            undo=True,
            wrap=tk.NONE,
            vertical_scrollbar_style=scrollbar_style("Vertical"),
            horizontal_scrollbar_style=scrollbar_style("Horizontal"),
            horizontal_scrollbar_class=ui_utils.AutoScrollbar,
            **text_frame_args
        )

        # TODO: propose_remove_line_numbers on paste??

        assert self._first_line_number is not None

        self._syntax_theme_change_binding = tk._default_root.bind(
            "<<SyntaxThemeChanged>>", self._reload_theme_options, True
        )

        self._reload_theme_options()
        self._gutter.bind("<Double-Button-1>", self._toggle_breakpoint, True)
        # self.text.tag_configure("breakpoint_line", background="pink")
        self._gutter.tag_configure("breakpoint", foreground="crimson")

        editor_font = tk.font.nametofont("EditorFont")
        spacer_font = editor_font.copy()
        spacer_font.configure(size=editor_font.cget("size") // 4)
        self._gutter.tag_configure("spacer", font=spacer_font)
        self._gutter.tag_configure("active", font="BoldEditorFont")
        self._gutter.tag_raise("spacer")

    def get_content(self):
        return self.text.get(
            "1.0", "end-1c"
        )  # -1c because Text always adds a newline itself

    def detect_encoding(self):
        encoding, _ = tokenize.detect_encoding(
            io.BytesIO(self.get_content().encode("ascii", errors="replace")).readline
        )
        return encoding

    def get_content_as_bytes(self):
        chars = self.get_content().replace("\r", "")
        if running_on_windows():
            chars = chars.replace("\n", "\r\n")

        return chars.encode(self.detect_encoding())

    def set_content(self, content, keep_undo=False):
        self.text.direct_delete("1.0", tk.END)
        self.text.direct_insert("1.0", content)
        self.update_gutter()

        if not keep_undo:
            self.text.edit_reset()

        self.text.event_generate("<<TextChange>>")

    def _toggle_breakpoint(self, event):
        index = "@%d,%d" % (event.x, event.y)
        start_index = index + " linestart"
        end_index = index + " lineend"

        if self.text.tag_nextrange("breakpoint_line", start_index, end_index):
            self.text.tag_remove("breakpoint_line", start_index, end_index)
        else:
            line_content = self.text.get(start_index, end_index).strip()
            if line_content and line_content[0] != "#":
                self.text.tag_add("breakpoint_line", start_index, end_index)

        self.update_gutter(clean=True)

    def compute_gutter_line(self, lineno):
        visual_line_number = self._first_line_number + lineno - 1
        linestart = str(visual_line_number) + ".0"

        yield from super().compute_gutter_line(lineno)
        yield " ", ("spacer",)

        if self.text.tag_nextrange(
            "breakpoint_line", linestart, linestart + " lineend"
        ):
            yield BREAKPOINT_SYMBOL, ("breakpoint",)
        else:
            yield " ", ()

    def select_range(self, text_range):
        self.text.tag_remove("sel", "1.0", tk.END)

        if text_range:
            if isinstance(text_range, int):
                # it's line number
                start = str(text_range - self._first_line_number + 1) + ".0"
                end = str(text_range - self._first_line_number + 1) + ".end"
            elif isinstance(text_range, TextRange):
                start = "%s.%s" % (
                    text_range.lineno - self._first_line_number + 1,
                    text_range.col_offset,
                )
                end = "%s.%s" % (
                    text_range.end_lineno - self._first_line_number + 1,
                    text_range.end_col_offset,
                )
            else:
                assert isinstance(text_range, tuple)
                start, end = text_range

            self.text.tag_add("sel", start, end)
            if isinstance(text_range, int):
                self.text.mark_set("insert", end)
            self.text.see("%s -1 lines" % start)

    def get_breakpoint_line_numbers(self):
        result = set()
        for num_line in self._gutter.get("1.0", "end").splitlines():
            if BREAKPOINT_SYMBOL in num_line:
                result.add(int(num_line.replace(BREAKPOINT_SYMBOL, "")))
        return result

    def get_selected_range(self):
        if self.text.has_selection():
            lineno, col_offset = map(int, self.text.index(tk.SEL_FIRST).split("."))
            end_lineno, end_col_offset = map(
                int, self.text.index(tk.SEL_LAST).split(".")
            )
        else:
            lineno, col_offset = map(int, self.text.index(tk.INSERT).split("."))
            end_lineno, end_col_offset = lineno, col_offset

        return TextRange(lineno, col_offset, end_lineno, end_col_offset)

    def destroy(self):
        super().destroy()
        tk._default_root.unbind(
            "<<SyntaxThemeChanged>>", self._syntax_theme_change_binding
        )

    def _reload_theme_options(self, event=None):
        super()._reload_theme_options(event)

        if "GUTTER" in _syntax_options:
            opts = _syntax_options["GUTTER"].copy()
            if "background" in opts and "selectbackground" not in opts:
                opts["selectbackground"] = opts["background"]
            if "foreground" in opts and "selectforeground" not in opts:
                opts["selectforeground"] = opts["foreground"]

            self._gutter.configure(opts)

            if "background" in _syntax_options["GUTTER"]:
                self._margin_line.configure(
                    background=_syntax_options["GUTTER"]["background"]
                )

        if "breakpoint" in _syntax_options:
            self._gutter.tag_configure("breakpoint", _syntax_options["breakpoint"])


def set_syntax_options(syntax_options):
    global _syntax_options
    _syntax_options = syntax_options

    assert tk._default_root is not None
    tk._default_root.event_generate("<<SyntaxThemeChanged>>")


def get_syntax_options_for_tag(tag, **base_options):
    global _syntax_options
    if tag in _syntax_options:
        base_options.update(_syntax_options[tag])
    return base_options



import tkinter as tk
from tkinter import ttk

from thonny import get_runner, get_workbench, misc_utils, ui_utils
from thonny.codeview import CodeView


class ShellMacroDialog(tk.Toplevel):
    def __init__(self, master):
        tk.Toplevel.__init__(self, master)

        self.title("Configure shell macro")
        if misc_utils.running_on_mac_os():
            self.configure(background="systemSheetBackground")
        self.transient(master)
        self.grab_set()  # to make it active
        # self.grab_release() # to allow eg. copy something from the editor

        self._create_widgets()

        self.bind("<Escape>", self._on_close, True)
        self.protocol("WM_DELETE_WINDOW", self._on_close)
        self.main_command_text.focus_set()

    def _create_widgets(self):

        bg = "#ffff99"
        banner_frame = tk.Frame(self, background=bg)
        banner_frame.grid(row=0, column=0, sticky="nsew")
        banner_frame.rowconfigure(0, weight=1)
        banner_frame.columnconfigure(0, weight=1)
        banner_text = tk.Label(
            banner_frame,
            text="These\nare\ninstructions asdfa afs fa sfasdf",
            background=bg,
            justify="left",
        )
        banner_text.grid(column=0, row=0, pady=10, padx=10, sticky="nsew")

        main_frame = ttk.Frame(self)
        main_frame.grid(row=1, column=0, sticky=tk.NSEW, padx=15, pady=15)
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)

        self.main_command_text = CodeView(main_frame, height=5)
        self.main_command_text.grid(column=0, row=1, sticky="nsew")
        # main_command_text["relief"] = "groove"

        main_frame.rowconfigure(1, weight=1)
        main_frame.columnconfigure(0, weight=1)

        button_frame = ttk.Frame(main_frame)
        button_frame.grid(row=2, column=0, sticky="nsew")

        run_button = ttk.Button(
            button_frame, text="Save and execute", command=self._save_exec
        )
        run_button.grid(row=0, column=1, sticky="nsew")
        ok_button = ttk.Button(button_frame, text="Save", command=self._save)
        ok_button.grid(row=0, column=2, sticky="nsew")
        cancel_button = ttk.Button(button_frame, text="Cancel", command=self._cancel)
        cancel_button.grid(row=0, column=3, sticky="nsew")
        button_frame.columnconfigure(0, weight=1)

    def _on_close(self, event=None):
        self.destroy()

    def _save_exec(self, event=None):
        self._save(event)
        execute_macro()

    def _save(self, event=None):
        source = self.main_command_text.text.get("1.0", "end")
        get_workbench().set_option("run.shell_macro_main", repr(source))
        self.destroy()

    def _cancel(self, event=None):
        self.destroy()


def show_dialog():
    dlg = ShellMacroDialog(get_workbench())
    ui_utils.show_dialog(dlg)


def execute_macro():
    if get_runner().is_waiting_toplevel_command():
        source = get_workbench().get_option("run.shell_macro_main")
        if source is not None:
            shell = get_workbench().show_view("ShellView")
            shell.submit_python_code(source.strip() + "\n")


def _load_plugin():
    get_workbench().set_default("run.shell_macro_main", None)
    get_workbench().add_command(
        "configure_shell_macro", "run", "Configure shell macro...", show_dialog
    )
    get_workbench().add_command(
        "execute_shell_macro",
        "run",
        "Execute shell macro",
        execute_macro,
        default_sequence="<F9>",
    )



import ast
from textwrap import dedent

from thonny.ast_utils import pretty


def test_pretty_without_end_markers():
    p = pretty(
        ast.parse(
            dedent(
                """
    age = int(input("Enter age: "))
    if age > 18:
        print("Hi")
    else:
        print("Hello!", end='')
        print("What's your name?")
    """
            ).strip()
        )
    )

    assert (
        p
        == """/=Module
    body=[...]
        0=Assign @ 1.0
            targets=[...]
                0=Name @ 1.0
                    id='age'
                    ctx=Store
            value=Call @ 1.6
                func=Name @ 1.6
                    id='int'
                    ctx=Load
                args=[...]
                    0=Call @ 1.10
                        func=Name @ 1.10
                            id='input'
                            ctx=Load
                        args=[...]
                            0=Str @ 1.16
                                s='Enter age: '
                        keywords=[]
                keywords=[]
        1=If @ 2.0
            test=Compare @ 2.3
                left=Name @ 2.3
                    id='age'
                    ctx=Load
                ops=[...]
                    0=Gt
                comparators=[...]
                    0=Num @ 2.9
                        n=18
            body=[...]
                0=Expr @ 3.4
                    value=Call @ 3.4
                        func=Name @ 3.4
                            id='print'
                            ctx=Load
                        args=[...]
                            0=Str @ 3.10
                                s='Hi'
                        keywords=[]
            orelse=[...]
                0=Expr @ 5.4
                    value=Call @ 5.4
                        func=Name @ 5.4
                            id='print'
                            ctx=Load
                        args=[...]
                            0=Str @ 5.10
                                s='Hello!'
                        keywords=[...]
                            0=keyword
                                arg='end'
                                value=Str @ 5.24
                                    s=''
                1=Expr @ 6.4
                    value=Call @ 6.4
                        func=Name @ 6.4
                            id='print'
                            ctx=Load
                        args=[...]
                            0=Str @ 6.10
                                s="What's your name?"
                        keywords=[]"""
    )



import tkinter as tk

from jedi import Script

from thonny import get_workbench
from thonny.ui_utils import control_is_pressed


def goto_definition(event):
    if not control_is_pressed(event.state):
        return

    assert isinstance(event.widget, tk.Text)
    text = event.widget

    source = text.get("1.0", "end")
    index = text.index("insert")
    index_parts = index.split(".")
    line, column = int(index_parts[0]), int(index_parts[1])
    # TODO: find current editor filename
    script = Script(source, line=line, column=column, path="")
    defs = script.goto_definitions()
    if len(defs) > 0:
        module_path = defs[0].module_path
        module_name = defs[0].module_name
        line = defs[0].line
        if module_path and line is not None:
            get_workbench().get_editor_notebook().show_file(module_path, line)
        elif module_name == "" and line is not None:  # current editor
            get_workbench().get_editor_notebook().get_current_editor().select_range(
                line
            )


def load_plugin() -> None:
    wb = get_workbench()
    wb.bind_class("CodeViewText", "<1>", goto_definition, True)



class OrderedDict:
    ""

    def clear():
        pass

    def copy():
        pass

    def fromkeys():
        pass

    def get():
        pass

    def items():
        pass

    def keys():
        pass

    def pop():
        pass

    def popitem():
        pass

    def setdefault():
        pass

    def update():
        pass

    def values():
        pass


class deque:
    ""

    def append():
        pass

    def popleft():
        pass


def namedtuple():
    pass



import sys
import runpy

if sys.executable.endswith("thonny.exe"):
    # otherwise some library may try to run its subprocess with thonny.exe
    sys.executable = sys.executable[:-len("thonny.exe")] + "pythonw.exe"

from thonny import launch

try:
    runpy.run_module("thonny.customize", run_name="__main__")
except ImportError:
    pass


launch()



from tkinter import ttk
from thonny.config_ui import ConfigurationPage
from thonny import get_workbench
from thonny.misc_utils import running_on_mac_os

class TerminalConfigurationPage(ConfigurationPage):
    def __init__(self, master):
        super().__init__(master)
        runscript_label = ttk.Label(self, text="Running current script in terminal:")
        runscript_label.grid(row=0, column=0, sticky="w")
        
        
        self.add_checkbox(
            "run.run_in_terminal_python_repl",
            "Present Python REPL after program ends",
            row=1,
            padx=(12,0)
        )
        
        self.add_checkbox(
            "run.run_in_terminal_keep_open",
            "Keep terminal window open after Python process ends",
            row=2,
            padx=(12,0)
        )
        
        exit_remark = " "
        if running_on_mac_os():
            exit_remark = ("NB! Automatic closing needs to be enabled in Terminal's settings\n"
                           + "(Profiles → Shell → When the shell exits)")
        remark_label = ttk.Label(self, text=exit_remark)
        remark_label.grid(row=3, column=0, sticky="w", padx=(12,0), pady=(0,10))


def load_plugin():
    get_workbench().add_configuration_page("Terminal", TerminalConfigurationPage)
    


import os

from thonny import get_workbench

_OPTION_NAME = "run.pgzero_mode"


def toggle_variable():
    var = get_workbench().get_variable(_OPTION_NAME)
    var.set(not var.get())
    update_environment()


def update_environment():
    if get_workbench().get_ui_mode() == "simple":
        os.environ["PGZERO_MODE"] = "auto"
    else:
        os.environ["PGZERO_MODE"] = str(get_workbench().get_option(_OPTION_NAME))


def load_plugin():
    get_workbench().set_default(_OPTION_NAME, False)
    get_workbench().add_command(
        "toggle_pgzero_mode",
        "run",
        "Pygame Zero mode",
        toggle_variable,
        flag_name=_OPTION_NAME,
        group=40,
    )
    update_environment()
